{
  version 1.0 first release
  version 1.1 fixed bug in destroy of TCounterModel (inherited Destroy was not called)

  for comments/bugs
  F.Bouwmans
  fbouwmans@spiditel.nl
}

unit ObservableComponent;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TUpdateEvent = procedure (arg: TObject) of object;
  TObserver = class;
  TObserverComponent = class;

  TObservableComponent = class(TComponent)
  private
    { Private declarations }
    Changed: Boolean;
    observerList: TList;
    observerComponentList: TList;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;

    procedure addObserver(observer: TObserver);
    procedure deleteObserver(observer: TObserver);

    procedure addObserverComponent(observer: TObserverComponent);
    procedure deleteObserverComponent(observer: TObserverComponent);

    procedure deleteAllObservers;
    procedure notifyObservers(arg: TObject);
    procedure setChanged;
    procedure clearChanged;
    function hasChanged: Boolean;
  published
    { Published declarations }
  end;

  TObserver = class(TObject)
  public
    constructor Create(model: TObservableComponent; EVENT: TUpdateEvent);
    destructor Destroy; override;
    procedure Update(arg: TObject);
    procedure deleteObservable;
  private
    notifyEvent: TUpdateEvent;
    observable: TObservableComponent;
  end;

  TObserverComponent = class(TComponent)
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy; override;

    procedure setObservable(model: TObservableComponent);
    procedure Update(arg: TObject);
    procedure deleteObservable;
  private
    notifyEvent: TUpdateEvent;
    FObservable: TObservableComponent;
  published
    { Published declarations }
    property Observable:TObservableComponent read FObservable write setObservable;
    property OnUpdate:TUpdateEvent read notifyEvent write notifyEvent;
  end;

procedure Register;

implementation

constructor TObserver.Create(model: TObservableComponent; EVENT: TUpdateEvent);
begin
  observable := model;
  notifyEvent := EVENT;
  observable.addObserver(self);
end;

destructor TObserver.Destroy;
begin
  if not (observable=nil) then
    observable.deleteObserver(self);
  inherited destroy;
end;

procedure TObserver.Update(arg: TObject);
begin
  if assigned(notifyEvent) then
    notifyEvent(arg);
end;
procedure TObserver.deleteObservable;
begin
  observable := nil;
end;


constructor TObserverComponent.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

procedure TObserverComponent.setObservable(model: TObservableComponent);
begin
  FObservable := model;
  if assigned(FObservable) then
    FObservable.addObserverComponent(self);
end;

destructor TObserverComponent.Destroy;
begin
  if assigned(FObservable) then
    FObservable.deleteObserverComponent(self);
  inherited destroy;
end;
procedure TObserverComponent.deleteObservable;
begin
  FObservable := nil;
end;
procedure TObserverComponent.Update(arg: TObject);
begin
  if assigned(notifyEvent) then notifyEvent(arg);
end;

constructor TObservableComponent.Create(AOwner: TComponent);
var I: Integer;
begin
  inherited Create(AOwner);
  observerList := TList.Create;
  observerComponentList := TList.Create;
  Changed := False;
end;

destructor TObservableComponent.Destroy;
var i : integer;
begin
  for I := 0 to observerList.Count - 1 do
    TObserver(observerList[I]).deleteObservable;
  for I := 0 to observerComponentList.Count - 1 do
    TObserverComponent(observerComponentList[I]).deleteObservable;

  observerList.Free;
  observerComponentList.Free;
  inherited Destroy;
end;

procedure TObservableComponent.addObserver(observer: TObserver);
begin
  observerList.Add(observer);
end;

procedure TObservableComponent.deleteObserver(observer: TObserver);
var I: Integer;
begin
  I := observerList.IndexOf(TObject(observer));
  if I >= 0 then
    observerList.Delete(I);
end;

procedure TObservableComponent.addObserverComponent(observer: TObserverComponent);
begin
  observerComponentList.Add(observer);
end;

procedure TObservableComponent.deleteObserverComponent(observer: TObserverComponent);
var I: Integer;
begin
  I := observerComponentList.IndexOf(TObject(observer));
  if I >= 0 then
    observerComponentList.Delete(I);
end;

procedure TObservableComponent.deleteAllObservers;
var I: Integer;
begin
  observerList.Clear;
  observerComponentList.CLear;
end;

procedure TObservableComponent.notifyObservers(arg: TObject);
var I: Integer;
begin
  for I := 0 to observerList.Count - 1 do
    TObserver(observerList[I]).Update(arg);
  for I := 0 to observerComponentList.Count - 1 do
    TObserverComponent(observerComponentList[I]).Update(arg);
end;

procedure TObservableComponent.setChanged;
begin
  Changed := True;
end;

procedure TObservableComponent.clearChanged;
begin
  Changed := False;
end;

function TObservableComponent.hasChanged: Boolean;
begin
  Result := Changed;
end;

procedure Register;
begin
  RegisterComponents('Mvc', [TObservableComponent,TObserverComponent]);
end;

end.
